@@ -4,9 +4,8 @@
 #
 ##############################
 # C O P Y R I G H T   N O T I C E
-#  Copyright (c) 2001-2002 by:
+#  Copyright (c) 2001-2006 by:
 #    * The MicroArray Gene Expression Database Society (MGED)
-#    * Rosetta Inpharmatics
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -28,14 +27,19 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+
+
 package Bio::MAGE::ArrayDesign;
 
 use strict;
+
+use base qw(Bio::MAGE::Base);
+
 use Carp;
-use Bio::MAGE::Base;
-require Exporter;
 use Tie::IxHash;
 
+use vars qw($__XML_LISTS);
+
 use Bio::MAGE::ArrayDesign::ArrayDesign;
 use Bio::MAGE::ArrayDesign::PhysicalArrayDesign;
 use Bio::MAGE::ArrayDesign::ZoneLayout;
@@ -47,29 +51,9 @@ use Bio::MAGE::ArrayDesign::DesignElementGroup;
 use Bio::MAGE::ArrayDesign::CompositeGroup;
 
 
-use vars qw(@ISA $VERSION @EXPORT_OK %EXPORT_TAGS $__MAGEML_LISTS);
-
-@ISA = qw(Bio::MAGE::Base Exporter);
-$VERSION = 20020902.6;
-
-sub initialize {
-  my $self = shift;
-  $self->reportergroup_list([]);
-  $self->compositegroup_list([]);
-  $self->arraydesign_list([]);
-
-  $self->mageml_lists([ReporterGroup=>$self->reportergroup_list,
-	CompositeGroup=>$self->compositegroup_list,
-	ArrayDesign=>$self->arraydesign_list]);
-  $self->tagname(q[ArrayDesign_package]);
-  return 1;
-}
-
-
-
 =head1 NAME
 
-Bio::MAGE::ArrayDesign - Container module for classes in the MAGE ArrayDesign package 
+Bio::MAGE::ArrayDesign - Container module for classes in the MAGE package: ArrayDesign
 
 =head1 SYNOPSIS
 
@@ -132,33 +116,122 @@ This method returns a list of non-fully qualified class names
 =cut
 
 sub classes {
-  return ('ArrayDesign',
-'PhysicalArrayDesign',
-'ZoneLayout',
-'ZoneGroup',
-'Zone',
-'ReporterGroup',
-'FeatureGroup',
-'DesignElementGroup',
-'CompositeGroup');
+  return ('ArrayDesign','PhysicalArrayDesign','ZoneLayout','ZoneGroup','Zone','ReporterGroup','FeatureGroup','DesignElementGroup','CompositeGroup');
 }
 
-=item $obj_ref = Bio::MAGE::ArrayDesign->new(%parameters)
 
-=item $obj_ref_copy = $obj_ref->new()
 
-The C<new()> method is the class constructor. If invoked with an
-existing object instance, it is a copy constructor.
 
-B<Return value>: It returns a reference to an object of class
-C<Bio::MAGE::ArrayDesign>.
+
+=item $obj = class->new(%parameters)
+
+The C<new()> method is the class constructor.
+
+B<Parameters>: if given a list of name/value parameters the
+corresponding slots, attributes, or associations will have their
+initial values set by the constructor.
+
+B<Return value>: It returns a reference to an object of the class.
 
 B<Side effects>: It invokes the C<initialize()> method if it is defined
 by the class.
 
 =cut
 
+#
+# code for new() inherited from Base.pm
+#
 
+=item @names = class->get_slot_names()
+
+The C<get_slot_names()> method is used to retrieve the name of all
+slots defined in a given class.
+
+B<NOTE>: the list of names does not include attribute or association
+names.
+
+B<Return value>: A list of the names of all slots defined for this class.
+
+B<Side effects>: none
+
+=cut
+
+#
+# code for get_slot_names() inherited from Base.pm
+#
+
+=item @name_list = get_attribute_names()
+
+returns the list of attribute data members for this class.
+
+=cut
+
+#
+# code for get_attribute_names() inherited from Base.pm
+#
+
+=item @name_list = get_association_names()
+
+returns the list of association data members for this class.
+
+=cut
+
+#
+# code for get_association_names() inherited from Base.pm
+#
+
+=item @class_list = get_superclasses()
+
+returns the list of superclasses for this class.
+
+=cut
+
+#
+# code for get_superclasses() inherited from Base.pm
+#
+
+=item @class_list = get_subclasses()
+
+returns the list of subclasses for this class.
+
+=cut
+
+#
+# code for get_subclasses() inherited from Base.pm
+#
+
+=item $name = class_name()
+
+Returns the full class name for this class.
+
+=cut
+
+#
+# code for class_name() inherited from Base.pm
+#
+
+=item $package_name = package_name()
+
+Returns the base package name (i.e. no 'namespace::') of the package
+that contains this class.
+
+=cut
+
+#
+# code for package_name() inherited from Base.pm
+#
+
+=item %assns = associations()
+
+returns the association meta-information in a hash where the keys are
+the association names and the values are C<Association> objects that
+provide the meta-information for the association.
+
+=cut
+
+#
+# code for associations() inherited from Base.pm
+#
 
 
 
@@ -170,143 +243,22 @@ by the class.
 
 =cut
 
-=item $array_ref = $arraydesign->getReporterGroup_list()
+=item $obj_copy = $obj->new()
 
-This method handles the list for the C<Bio::MAGE::ArrayDesign::ReporterGroup> class. It
-returns a reference to an array of the class objects that have been
-associated with the package instance.
+When invoked with an existing object reference and not a class name,
+the C<new()> method acts as a copy constructor - with the new object's
+initial values set to be those of the existing object.
 
-This is useful when retrieving data from parsed MAGE-ML file.
+B<Parameters>: No input parameters  are used in the copy  constructor,
+the initial values are taken directly from the object to be copied.
+
+B<Return value>: It returns a reference to an object of the class.
+
+B<Side effects>: It invokes the C<initialize()> method if it is defined
+by the class.
 
 =cut
 
-sub getReporterGroup_list {
-  my $self = shift;
-  return $self->reportergroup_list();
-}
-
-=item $arraydesign->addReporterGroup(@vals)
-
-This method is an interface for adding C<Bio::MAGE::ArrayDesign::ReporterGroup> objects to
-the C<reportergroup_list> list. It is generally used by generic methods such
-as those in the XMLWriter.
-
-Input parameters: the list of values C<@vals> to add to the owner
-association. B<NOTE>: submitting a single value is permitted.
-
-Return value: none
-
-Side effects: none
-
-Exceptions: will call C<croak()> if no input parameters are specified
-, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::ReporterGroup>
-
-=cut
-
-sub addReporterGroup {
-  my $self = shift;
-  croak("Bio::MAGE::ArrayDesign::addReporterGroup: no arguments passed to setter")
-    unless scalar @_;
-  foreach my $val (@_) {
-    croak("Bio::MAGE::ArrayDesign::addReporterGroup: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::ReporterGroup")
-      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::ReporterGroup');
-  }
-
-  push(@{$self->reportergroup_list},@_);
-}
-
-=item $array_ref = $arraydesign->getCompositeGroup_list()
-
-This method handles the list for the C<Bio::MAGE::ArrayDesign::CompositeGroup> class. It
-returns a reference to an array of the class objects that have been
-associated with the package instance.
-
-This is useful when retrieving data from parsed MAGE-ML file.
-
-=cut
-
-sub getCompositeGroup_list {
-  my $self = shift;
-  return $self->compositegroup_list();
-}
-
-=item $arraydesign->addCompositeGroup(@vals)
-
-This method is an interface for adding C<Bio::MAGE::ArrayDesign::CompositeGroup> objects to
-the C<compositegroup_list> list. It is generally used by generic methods such
-as those in the XMLWriter.
-
-Input parameters: the list of values C<@vals> to add to the owner
-association. B<NOTE>: submitting a single value is permitted.
-
-Return value: none
-
-Side effects: none
-
-Exceptions: will call C<croak()> if no input parameters are specified
-, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::CompositeGroup>
-
-=cut
-
-sub addCompositeGroup {
-  my $self = shift;
-  croak("Bio::MAGE::ArrayDesign::addCompositeGroup: no arguments passed to setter")
-    unless scalar @_;
-  foreach my $val (@_) {
-    croak("Bio::MAGE::ArrayDesign::addCompositeGroup: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::CompositeGroup")
-      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::CompositeGroup');
-  }
-
-  push(@{$self->compositegroup_list},@_);
-}
-
-=item $array_ref = $arraydesign->getArrayDesign_list()
-
-This method handles the list for the C<Bio::MAGE::ArrayDesign::ArrayDesign> class. It
-returns a reference to an array of the class objects that have been
-associated with the package instance.
-
-This is useful when retrieving data from parsed MAGE-ML file.
-
-=cut
-
-sub getArrayDesign_list {
-  my $self = shift;
-  return $self->arraydesign_list();
-}
-
-=item $arraydesign->addArrayDesign(@vals)
-
-This method is an interface for adding C<Bio::MAGE::ArrayDesign::ArrayDesign> objects to
-the C<arraydesign_list> list. It is generally used by generic methods such
-as those in the XMLWriter.
-
-Input parameters: the list of values C<@vals> to add to the owner
-association. B<NOTE>: submitting a single value is permitted.
-
-Return value: none
-
-Side effects: none
-
-Exceptions: will call C<croak()> if no input parameters are specified
-, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::ArrayDesign>
-
-=cut
-
-sub addArrayDesign {
-  my $self = shift;
-  croak("Bio::MAGE::ArrayDesign::addArrayDesign: no arguments passed to setter")
-    unless scalar @_;
-  foreach my $val (@_) {
-    croak("Bio::MAGE::ArrayDesign::addArrayDesign: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::ArrayDesign")
-      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::ArrayDesign');
-  }
-
-  push(@{$self->arraydesign_list},@_);
-}
-
-
-
 #
 # code for new() inherited from Base.pm
 #
@@ -330,7 +282,7 @@ does not define.
 # code for set_slots() inherited from Base.pm
 #
 
-=item $obj->get_slots(@name_list)
+=item @obj_list = $obj->get_slots(@name_list)
 
 The C<get_slots()> method is used to get the values of a number of
 slots at the same time.
@@ -375,36 +327,20 @@ B<Side effects>: none
 # code for get_slot() inherited from Base.pm
 #
 
-=item @names = $obj->get_slot_names()
-
-The C<get_slot_names()> method is used to retrieve the name of all
-slots defined for a given object.
-
-B<Return value>: a single slot value, or undef if the slot has not been
-initialized.
-
-B<Side effects>: none
-
-=cut
-
-#
-# code for get_slot_names() inherited from Base.pm
-#
 
 
+=item $val = $arraydesign->xml_lists()
 
+=item $inval = $arraydesign->xml_lists($inval)
 
-=item $val = $arraydesign->reportergroup_list()
+This is the unified setter/getter method for the xml_lists slot.
 
-=item $val = $arraydesign->reportergroup_list($val)
-
-This is the unified setter/getter method for the reportergroup_list slot. If
-C<$val> is specified, the setter method is invoked, with no
+If C<$inval> is specified, the setter method is invoked, with no
 parameters, the getter method is invoked.
 
-Input parameters: the optional C<$val> will invoke the setter method.
+Input parameters: the optional C<$inval> will invoke the setter method.
 
-Return value: for both setter and getter the current value of the reportergroup_list
+Return value: for both setter and getter the current value of the xml_lists
 slot
 
 Side effects: none
@@ -413,104 +349,29 @@ Exceptions: none
 
 =cut
 
-sub reportergroup_list {
+
+sub xml_lists {
   my $self = shift;
   if (@_) {
-    $self->{__REPORTERGROUP_LIST} = shift;
+    $self->{__XML_LISTS} = shift;
   }
-  return $self->{__REPORTERGROUP_LIST};
+  return $self->{__XML_LISTS};
 }
 
-=item $val = $arraydesign->compositegroup_list()
 
-=item $val = $arraydesign->compositegroup_list($val)
 
-This is the unified setter/getter method for the compositegroup_list slot. If
-C<$val> is specified, the setter method is invoked, with no
-parameters, the getter method is invoked.
 
-Input parameters: the optional C<$val> will invoke the setter method.
-
-Return value: for both setter and getter the current value of the compositegroup_list
-slot
-
-Side effects: none
-
-Exceptions: none
-
-=cut
-
-sub compositegroup_list {
-  my $self = shift;
-  if (@_) {
-    $self->{__COMPOSITEGROUP_LIST} = shift;
-  }
-  return $self->{__COMPOSITEGROUP_LIST};
-}
-
-=item $val = $arraydesign->arraydesign_list()
-
-=item $val = $arraydesign->arraydesign_list($val)
-
-This is the unified setter/getter method for the arraydesign_list slot. If
-C<$val> is specified, the setter method is invoked, with no
-parameters, the getter method is invoked.
-
-Input parameters: the optional C<$val> will invoke the setter method.
-
-Return value: for both setter and getter the current value of the arraydesign_list
-slot
-
-Side effects: none
-
-Exceptions: none
-
-=cut
-
-sub arraydesign_list {
-  my $self = shift;
-  if (@_) {
-    $self->{__ARRAYDESIGN_LIST} = shift;
-  }
-  return $self->{__ARRAYDESIGN_LIST};
-}
-
-=item $val = $arraydesign->mageml_lists()
-
-=item $val = $arraydesign->mageml_lists($val)
-
-This is the unified setter/getter method for the mageml_lists slot. If
-C<$val> is specified, the setter method is invoked, with no
-parameters, the getter method is invoked.
-
-Input parameters: the optional C<$val> will invoke the setter method.
-
-Return value: for both setter and getter the current value of the mageml_lists
-slot
-
-Side effects: none
-
-Exceptions: none
-
-=cut
-
-sub mageml_lists {
-  my $self = shift;
-  if (@_) {
-    $self->{__MAGEML_LISTS} = shift;
-  }
-  return $self->{__MAGEML_LISTS};
-}
 
 =item $val = $arraydesign->tagname()
 
-=item $val = $arraydesign->tagname($val)
+=item $inval = $arraydesign->tagname($inval)
 
-This is the unified setter/getter method for the tagname slot. If
-C<$val> is specified, the setter method is invoked, with no
+This is the unified setter/getter method for the tagname slot.
+
+If C<$inval> is specified, the setter method is invoked, with no
 parameters, the getter method is invoked.
 
-Input parameters: the optional C<$val> will invoke the setter method.
+Input parameters: the optional C<$inval> will invoke the setter method.
 
 Return value: for both setter and getter the current value of the tagname
 slot
@@ -521,6 +382,7 @@ Exceptions: none
 
 =cut
 
+
 sub tagname {
   my $self = shift;
   if (@_) {
@@ -532,27 +394,283 @@ sub tagname {
 
 
 
-=back
+
+=item $val = $arraydesign->reportergroup_list()
+
+=item $inval = $arraydesign->reportergroup_list($inval)
+
+This is the unified setter/getter method for the reportergroup_list slot.
+
+If C<$inval> is specified, the setter method is invoked, with no
+parameters, the getter method is invoked.
+
+Input parameters: the optional C<$inval> will invoke the setter method.
+
+Return value: for both setter and getter the current value of the reportergroup_list
+slot
+
+Side effects: none
+
+Exceptions: none
 
 =cut
 
-sub register {
-  my ($self,$obj) = @_;
 
-  # should we have the identifier checking code here??
-  my %mageml_lists = @{$self->mageml_lists()};
-  my $list_ref;
-  foreach my $class (keys %mageml_lists) {
-    if ($obj->isa("Bio::MAGE::ArrayDesign::$class")) {
-      $list_ref = $mageml_lists{$class};
-      last;
-    }
+sub reportergroup_list {
+  my $self = shift;
+  if (@_) {
+    $self->{__REPORTERGROUP_LIST} = shift;
   }
-  return unless defined $list_ref;
-  push(@{$list_ref}, $obj);
+  return $self->{__REPORTERGROUP_LIST};
 }
 
-sub obj2mageml {
+
+
+
+
+=item $val = $arraydesign->compositegroup_list()
+
+=item $inval = $arraydesign->compositegroup_list($inval)
+
+This is the unified setter/getter method for the compositegroup_list slot.
+
+If C<$inval> is specified, the setter method is invoked, with no
+parameters, the getter method is invoked.
+
+Input parameters: the optional C<$inval> will invoke the setter method.
+
+Return value: for both setter and getter the current value of the compositegroup_list
+slot
+
+Side effects: none
+
+Exceptions: none
+
+=cut
+
+
+sub compositegroup_list {
+  my $self = shift;
+  if (@_) {
+    $self->{__COMPOSITEGROUP_LIST} = shift;
+  }
+  return $self->{__COMPOSITEGROUP_LIST};
+}
+
+
+
+
+
+=item $val = $arraydesign->arraydesign_list()
+
+=item $inval = $arraydesign->arraydesign_list($inval)
+
+This is the unified setter/getter method for the arraydesign_list slot.
+
+If C<$inval> is specified, the setter method is invoked, with no
+parameters, the getter method is invoked.
+
+Input parameters: the optional C<$inval> will invoke the setter method.
+
+Return value: for both setter and getter the current value of the arraydesign_list
+slot
+
+Side effects: none
+
+Exceptions: none
+
+=cut
+
+
+sub arraydesign_list {
+  my $self = shift;
+  if (@_) {
+    $self->{__ARRAYDESIGN_LIST} = shift;
+  }
+  return $self->{__ARRAYDESIGN_LIST};
+}
+
+
+
+
+
+
+sub initialize {
+  my $self = shift;
+
+  $self->reportergroup_list([]);
+  $self->compositegroup_list([]);
+  $self->arraydesign_list([]);
+
+  $self->xml_lists([ReporterGroup=>$self->reportergroup_list(), CompositeGroup=>$self->compositegroup_list(), ArrayDesign=>$self->arraydesign_list()]);
+
+  $self->tagname(q[ArrayDesign_package]);
+  return 1;
+}
+
+
+=item $array_ref = $arraydesign->getReporterGroup_list()
+
+This method handles the list for the C<Bio::MAGE::ArrayDesign::ReporterGroup> class. It
+returns a reference to an array of the class objects that have been
+associated with the package instance.
+
+This is useful when retrieving data from parsed MAGE-ML file.
+
+=cut
+
+sub getReporterGroup_list {
+  my $self = shift;
+  return $self->reportergroup_list();
+}
+
+=item $arraydesign->addReporterGroup(@vals)
+
+This method is an interface for adding C<Bio::MAGE::ArrayDesign::ReporterGroup> objects to
+the C<reportergroup_list> list. It is generally used by generic methods such
+as those in the XMLWriter.
+
+Input parameters: the list of values C<@vals> to add to the owner
+association. B<NOTE>: submitting a single value is permitted.
+
+Return value: none
+
+Side effects: none
+
+Exceptions: will call C<croak()> if no input parameters are specified
+, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::ReporterGroup>
+
+=cut
+
+sub addReporterGroup {
+  my $self = shift;
+  croak(__PACKAGE__ . "::addReporterGroup: no arguments passed to setter")
+    unless scalar @_;
+  foreach my $val (@_) {
+    croak(__PACKAGE__ . "::addReporterGroup: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::ReporterGroup")
+      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::ReporterGroup');
+  }
+
+  push(@{$self->reportergroup_list},@_);
+}
+
+
+=item $array_ref = $arraydesign->getCompositeGroup_list()
+
+This method handles the list for the C<Bio::MAGE::ArrayDesign::CompositeGroup> class. It
+returns a reference to an array of the class objects that have been
+associated with the package instance.
+
+This is useful when retrieving data from parsed MAGE-ML file.
+
+=cut
+
+sub getCompositeGroup_list {
+  my $self = shift;
+  return $self->compositegroup_list();
+}
+
+=item $arraydesign->addCompositeGroup(@vals)
+
+This method is an interface for adding C<Bio::MAGE::ArrayDesign::CompositeGroup> objects to
+the C<compositegroup_list> list. It is generally used by generic methods such
+as those in the XMLWriter.
+
+Input parameters: the list of values C<@vals> to add to the owner
+association. B<NOTE>: submitting a single value is permitted.
+
+Return value: none
+
+Side effects: none
+
+Exceptions: will call C<croak()> if no input parameters are specified
+, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::CompositeGroup>
+
+=cut
+
+sub addCompositeGroup {
+  my $self = shift;
+  croak(__PACKAGE__ . "::addCompositeGroup: no arguments passed to setter")
+    unless scalar @_;
+  foreach my $val (@_) {
+    croak(__PACKAGE__ . "::addCompositeGroup: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::CompositeGroup")
+      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::CompositeGroup');
+  }
+
+  push(@{$self->compositegroup_list},@_);
+}
+
+
+=item $array_ref = $arraydesign->getArrayDesign_list()
+
+This method handles the list for the C<Bio::MAGE::ArrayDesign::ArrayDesign> class. It
+returns a reference to an array of the class objects that have been
+associated with the package instance.
+
+This is useful when retrieving data from parsed MAGE-ML file.
+
+=cut
+
+sub getArrayDesign_list {
+  my $self = shift;
+  return $self->arraydesign_list();
+}
+
+=item $arraydesign->addArrayDesign(@vals)
+
+This method is an interface for adding C<Bio::MAGE::ArrayDesign::ArrayDesign> objects to
+the C<arraydesign_list> list. It is generally used by generic methods such
+as those in the XMLWriter.
+
+Input parameters: the list of values C<@vals> to add to the owner
+association. B<NOTE>: submitting a single value is permitted.
+
+Return value: none
+
+Side effects: none
+
+Exceptions: will call C<croak()> if no input parameters are specified
+, or if any of the objects in @vals is not a subclass of class C<Bio::MAGE::ArrayDesign::ArrayDesign>
+
+=cut
+
+sub addArrayDesign {
+  my $self = shift;
+  croak(__PACKAGE__ . "::addArrayDesign: no arguments passed to setter")
+    unless scalar @_;
+  foreach my $val (@_) {
+    croak(__PACKAGE__ . "::addArrayDesign: wrong type: " . ref($val) . " expected Bio::MAGE::ArrayDesign::ArrayDesign")
+      unless UNIVERSAL::isa($val,'Bio::MAGE::ArrayDesign::ArrayDesign');
+  }
+
+  push(@{$self->arraydesign_list},@_);
+}
+
+
+=item $arraydesign->obj2xml($writer)
+
+Write out this object, and all sub-objects, as XML using the supplied
+$writer to actually do the XML formatting.
+
+Input parameters: $writer must be an XML writer, e.g. an instance of
+Bio::MAGE::XML::Writer. It must have methods: write_start_tag(),
+write_end_tag(), and obj2xml().
+
+Return value: none
+
+Side effects: all writing is delegated to the $writer - it's
+write_start_tag() and write_end_tag() methods are invoked with the
+appropriate data, and all class sub-objects of the C<Bio::MAGE::ArrayDesign> instance will have their obj2xml() methods
+invoked in turn. By allowing the $writer to do the actual formatting
+of the output XML, it enables the user to precisely control the
+format.
+
+Exceptions: will call C<croak()> if no identifier has been set for the
+C<Bio::MAGE::ArrayDesign> instance.
+
+=cut
+
+sub obj2xml {
   my ($self,$writer) = @_;
 
   my $empty = 0;
@@ -560,16 +678,17 @@ sub obj2mageml {
   $writer->write_start_tag($tag,$empty);
 
   # we use IxHash because we need to preserve insertion order
-  tie my %list_hash, 'Tie::IxHash', @{$self->mageml_lists()};
+  tie my %list_hash, 'Tie::IxHash', @{$self->xml_lists()};
   foreach my $list_name (keys %list_hash) {
     if (scalar @{$list_hash{$list_name}}) {
       my $tag = $list_name . '_assnlist';
       $writer->write_start_tag($tag,$empty);
       foreach my $obj (@{$list_hash{$list_name}}) {
-	# this may seem a little odd, but the writer knows how to write out 
-	# the objects. Create a subclass of $OPTIONS{namespace}::XMLWriter to 
-	# modify the output of the obj2mageml process
-	$writer->obj2mageml($obj);
+	# this may seem a little odd, but the writer knows how to
+	# write out the objects - this allows you to create your own
+	# subclass of Bio::MAGE::XML::Writer and modify
+	# the output of the obj2xml process
+	$writer->obj2xml($obj);
       }
       $writer->write_end_tag($tag);
     }
@@ -580,48 +699,155 @@ sub obj2mageml {
 
 }
 
+=item $arraydesign->register($obj)
 
-###############################################################################
-# obj2database
-###############################################################################
-sub obj2database {
-  my ($self,$writer) = @_;
+Store an object for later writing as XML.
 
-  my $empty = 0;
-  my $tag = $self->tagname();
-  #$writer->write_start_tag($tag,$empty);
+Input parameters: object to be added to the list of registered objects.
 
-  # we use IxHash because we need to preserve insertion order
-  tie my %list_hash, 'Tie::IxHash', @{$self->mageml_lists()};
-  foreach my $list_name (keys %list_hash) {
-    if (scalar @{$list_hash{$list_name}}) {
-      my $tag = $list_name . '_assnlist';
-      #$writer->write_start_tag($tag,$empty);
-      foreach my $obj (@{$list_hash{$list_name}}) {
-	# this may seem a little odd, but the writer knows how to write out
-	# the objects. Create a subclass of Bio::MAGE::XMLWriter to
-	# modify the output of the obj2mageml process
-	$writer->obj2database($obj);
-      }
-      #$writer->write_end_tag($tag);
+Return value: none
+
+Side effects: if $obj needs to be stored by this class, a reference
+will be stored in the correct XML list for this class.
+
+Exceptions: none
+
+=cut
+
+sub register {
+  my ($self,$obj) = @_;
+
+  # should we have the identifier checking code here??
+  my %xml_lists = @{$self->xml_lists()};
+  my $list_ref;
+  foreach my $class (keys %xml_lists) {
+    if ($obj->isa("Bio::MAGE::ArrayDesign::$class")) {
+      $list_ref = $xml_lists{$class};
+      last;
     }
   }
 
-  # and we're done
-  #$writer->write_end_tag($tag);
-
+  return unless defined $list_ref;
+  push(@{$list_ref}, $obj);
 }
 
 
 
 
+
+=back
+
+=head1 SLOTS, ATTRIBUTES, AND ASSOCIATIONS
+
+In the Perl implementation of MAGE-OM classes, there are
+three types of class data members: C<slots>, C<attributes>, and
+C<associations>.
+
+=head2 SLOTS
+
+This API uses the term C<slot> to indicate a data member of the class
+that was not present in the UML model and is used for mainly internal
+purposes - use only if you understand the inner workings of the
+API. Most often slots are used by generic methods such as those in the
+XML writing and reading classes.
+
+Slots are implemented using unified getter/setter methods:
+
+=over
+
+=item $var = $obj->slot_name();
+
+Retrieves the current value of the slot.
+
+=item $new_var = $obj->slot_name($new_var);
+
+Store $new_var in the slot - the return value is also $new_var.
+
+=item @names = $obj->get_slot_names()
+
+Returns the list of all slots in the class.
+
+=back
+
+B<DATA CHECKING>: No data type checking is made for these methods.
+
+=head2 ATTRIBUTES AND ASSOCIATIONS
+
+The terms C<attribute> and C<association> indicate data members of the
+class that were specified directly from the UML model.
+
+In the Perl implementation of MAGE-OM classes,
+association and attribute accessors are implemented using three
+separate methods:
+
+=over
+
+=item get*
+
+Retrieves the current value.
+
+B<NOTE>: For associations, if the association has list cardinality, an
+array reference is returned.
+
+B<DATA CHECKING>: Ensure that no argument is provided.
+
+=item set*
+
+Sets the current value, B<replacing> any existing value.
+
+B<NOTE>: For associations, if the association has list cardinality,
+the argument must be an array reference. Because of this, you probably
+should be using the add* methods.
+
+B<DATA CHECKING>: For attributes, ensure that a single value is
+provided as the argument. For associations, if the association has
+list cardinality, ensure that the argument is a reference to an array
+of instances of the correct MAGE-OM class, otherwise
+ensure that there is a single argument of the correct MAGE-OM class.
+
+=item add*
+
+B<NOTE>: Only present in associations with list cardinality. 
+
+Appends a list of objects to any values that may already be stored
+in the association.
+
+B<DATA CHECKING>: Ensure that all arguments are of the correct MAGE-OM class.
+
+=back
+
+=head2 GENERIC METHODS
+
+The unified base class of all MAGE-OM classes, C<Bio::MAGE::Base>, provides a set of generic methods that
+will operate on slots, attributes, and associations:
+
+=over
+
+=item $val = $obj->get_slot($name)
+
+=item \@list_ref = $obj->get_slots(@name_list);
+
+=item $val = $obj->set_slot($name,$val)
+
+=item $obj->set_slots(%parameters)
+
+=item $obj->set_slots(\@name_list, \@value_list)
+
+See elsewhere in this page for a detailed description of these
+methods.
+
+=back
+
+=cut
+
+
 =head1 BUGS
 
-Please send bug reports to mged-mage@lists.sf.net
+Please send bug reports to the project mailing list: (mged-mage 'at' lists 'dot' sf 'dot' net)
 
 =head1 AUTHOR
 
-Jason E. Stewart (www.openinformatics.com)
+Jason E. Stewart (jasons 'at' cpan 'dot' org)
 
 =head1 SEE ALSO
 
@@ -629,9 +855,6 @@ perl(1).
 
 =cut
 
-
-#
-# End the module by returning a true value
-#
+# all perl modules must be true...
 1;
 
